import sys

import hifive_filler.field as field
import hifive_filler.parse as parse
import hifive_filler.token as token
import hifive_filler.placement as placement

def parse_field(player_number, field):
	incoming = input()
	(row_count, column_count) = parse.parse_field_meta(incoming)
	field.update_dimensions(row_count, column_count)
	_ = input() # drop column number row
	for row_index in range(0,field.row_count):
		incoming = input()
		row_as_string = parse.split_field_input_line(incoming)[1]
		(_, player_one, player_two) = parse.parse_field_line(row_as_string, row_index)
		if player_number == 1:
			field.player.update_content(player_one)
			field.opponent.update_content(player_two)
		else:
			field.player.update_content(player_two)
			field.opponent.update_content(player_one)
	# print('current player field is',field.player.content)

def parse_token(token):
	incoming = input()
	(row_count, column_count) = parse.parse_field_meta(incoming)
	token.update_dimensions(row_count, column_count)
	token.reset_values()
	for row_index in range(0,token.row_count):
		incoming = input()
		(token_res, _, _) = parse.parse_field_line(incoming, row_index)
		token.shape.update_content(token_res)
	# print('token is',token.shape.content)
	token.trim()
	# print('token is',token.shape.content)

def parse_input(player_number, field, token):
	parse_field(player_number, field)
	parse_token(token)

def place_token(field, token):
	possibilites = placement.find_possible_placements(field, token)
	could_place = (len(possibilites) > 0)
	if (len(possibilites) > 0):
		return (True, possibilites[0][0], possibilites[0][1])
	return (False, 0, 0)

def main() -> None:
	try:
		player_number = parse.parse_player(input())[0]
		f = field.Field()
		t = token.Token()
		could_place = True
		while could_place:
			parse_input(player_number, f, t)
			(could_place, x, y) = place_token(f, t)
			if could_place:
				print(f"{y} {x}")
			else:
				print("0 0")
	except Exception as e:
		sys.stderr.write(e)
		exit(1)

if __name__ == "__main__":
	main()



class Shape:
	def __init__(self):
		self.content = []
	
	def update_dimensions(self, row_count, column_count):
		self.add_dimensions(row_count, column_count)
		self.drop_dimensions(row_count, column_count)
	
	def add_dimensions(self, row_count, column_count):
		while len(self.content) < row_count:
			self.content.append([])
		for row_index in range(0,row_count):
			while len(self.content[row_index]) < column_count:
				self.content[row_index].append(False)
	
	def drop_dimensions(self, row_count, column_count):
		while row_count < len(self.content):
			self.content.pop()
		for row_index in range(0,row_count):
			while column_count < len(self.content[row_index]):
				self.content[row_index].pop()
	
	def update_content(self, coordinate_list):
		i = 0
		while i < len(coordinate_list):
			self.content[coordinate_list[i][0]][coordinate_list[i][1]] = True
			i += 1
	
	def get_max_dims(self):
		i = 0
		j = 0
		max_i = 0
		max_j = 0
		# print('token shape y length', len(self.content))
		while i < len(self.content):
			j = 0
			# print('token shape x length', len(self.content[i]))
			while j < len(self.content[i]):
				# print('y x', i, j)
				if self.content[i][j]:
					# print('true')
					if i > max_i:
						max_i = i
					if j > max_j:
						max_j = j
				j += 1
			i += 1
		return (max_i+1, max_j+1)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['hifive_filler']

package_data = \
{'': ['*']}

entry_points = \
{'console_scripts': ['hifive_filler = hifive_filler.main:main']}

setup_kwargs = {
    'name': 'hifive-filler',
    'version': '0.2.31',
    'description': '',
    'long_description': None,
    'author': 'yusata',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<3.10',
}


setup(**setup_kwargs)
